// --------------------------------------------------*- C++ -*---------------
//  Copyright (C) 1990-2007 by ILOG.
//  All Rights Reserved.
//
//  N O T I C E
//
//  THIS MATERIAL IS CONSIDERED A TRADE SECRET BY ILOG.
//  UNAUTHORIZED ACCESS, USE, REPRODUCTION OR DISTRIBUTION IS PROHIBITED.
// --------------------------------------------------------------------------
#ifndef __CONCERT_iloallocatorH
#define __CONCERT_iloallocatorH

#if !defined(__CONCERT_ilosysH)
# include <ilconcert/ilosys.h>
#endif

class IloMemory;

#ifdef AFCDEBUG
#define ILOALLOCCPP
#endif


//---------------------------------------------------------------------------
class IloAllocationList {

public:

	IloAllocationList();
	~IloAllocationList();
	
	void* alloc( IloInt size );
	void free( void* block );

	IloInt getMemoryUsage();

	void end();

private:

	struct Cell {
		IloInt _size;
		Cell* _previous;
		Cell* _next;
	};

	void free( Cell* cell );

	void* _first;
	IloInt _usedMemory;
};

//---------------------------------------------------------------------------
class IloFixedSizeAllocatorI {
private:
  struct Cell { Cell* _next; };
  IloBool _useIt;               // useful for purify the code
  const IloInt _nbBytesPerCell; // size of a cell to allocate in BYTES
  Cell* _firstCell;             // first free cell
  Cell* _toFree;                // what has actually been allocated
  void addFree(Cell *q) {
    assert(_useIt);
    q->_next=_toFree; _toFree=q;
  }
  void addCell(Cell *q) {
    assert(_useIt);
    q->_next=_firstCell; _firstCell=q;
  }
  IloInt _usedMemory;
  IloInt _totalMemory;
  IloInt getSize() const { return _nbBytesPerCell; }

  IloAllocationList* _allocationList;

public:
  IloBool useIt() const        { return _useIt; }
  void setUseIt(IloBool useIt) { 
	  _useIt=useIt; 
  	  if( !_useIt && 0 == _allocationList ) {
		ILOPROTECTNEW( _allocationList, new IloAllocationList );
	  }
  }
  IloInt getMemoryUsage() const      { return _usedMemory; }
  IloInt getTotalMemoryUsage() const { return _totalMemory; }
  IloFixedSizeAllocatorI(const IloInt nbBytesPerCell);
  ~IloFixedSizeAllocatorI();
#ifdef ILOALLOCCPP
  void* allocate(class IloMemory *mem);
#else
  void* allocate(class IloMemory *mem) {
    assert(_firstCell == 0 || _useIt);

	if( _useIt ) {
		Cell *p = _firstCell;
		if (!p)
		  p = allocateBlock(mem); // modifies _firstCell, or throws exception
		else
		  _firstCell = p->_next;
	    _usedMemory += _nbBytesPerCell;
	    return (void*)p;
	} else {
		assert(_allocationList!=0);
		void* p = _allocationList->alloc( _nbBytesPerCell );
		_totalMemory += _nbBytesPerCell;
		_usedMemory += _nbBytesPerCell;
		return p;
	}
  }
#endif
  void deallocate(void*);
  Cell* allocateBlock(IloMemory *mem);
};

class ILO_EXPORTED IloMemory {
private:
  enum { ILO_NB_ALLOC = 60 };
  IloFixedSizeAllocatorI* _array[ILO_NB_ALLOC];
  IloFixedSizeAllocatorI* getAllocator(IloInt s) const { return _array[s]; }
  IloInt getIndex(IloInt s) const;
  friend class IloFixedSizeAllocatorI;
public:
  ~IloMemory();
  IloMemory(IloBool);
  IloBool useIt() const { return _array[OL]->useIt(); }
  void setUseIt(IloBool useIt) const;
  void* alloc(const IloInt s) {
    return (s != 0) ? getAllocator(getIndex(s))->allocate(this) : (void *)0;
  }
  void free(void* p, const IloInt s) {
    if (p != 0) getAllocator(getIndex(s))->deallocate(p);
  }
  IloInt getTotalMemoryUsage() const;
  IloInt getMemoryUsage() const;

  void snapshot();
};

inline void* operator new (size_t sz, const IloMemory* h) {
  return ((IloMemory*)h)->alloc(sz);
}
#ifdef ILODELETEOPERATOR
inline void operator delete(void*, const IloMemory*) { }
#endif


//---------------------------------------------------------------------------
//  Stack Allocator
// --------------------------------------------------------------------------
class ILO_EXPORTED IloAllocationStack {
private:
  class Part {
    friend class IloAllocationStack;
  private:
    IloAny* _data;      // address of the first element
    IloInt  _size;      // Of Words
    IloInt _firstIndex; // index of the first element
    Part*  _prevPart; 
    Part*  _nextPart;
  public:
    Part();
    Part (IloUInt size, IloInt first=0, Part* prev=0);
    ~Part();
    IloInt getSize(); // bytes
  };

private:
  Part*   _currentPart;
  Part*   _firstPart;
  IloAny* _top; // address of current top
  IloInt  _topIndex; // index of current top
  IloInt  _size;     // size of the stack
  IloInt  _lost;

  IloBool isBelowBeginOfCurrentPart(IloInt index) {
    IloInt currentIndex = _currentPart->_firstIndex;
    return (IloBool)(currentIndex && index <= currentIndex);
  }
  IloBool isStrictlyBelowBeginOfCurrentPart(IloInt index) {
    IloInt currentIndex = _currentPart->_firstIndex;
    return (IloBool)(currentIndex && index < currentIndex);
  }
  IloBool isAboveEndOfCurrentPart(IloInt index) {
    // go to next part only if _top is really used
    return (IloBool)(index >=
		     (_currentPart->_firstIndex + _currentPart->_size));
  }
  void safeGoToNextPart(IloUInt size);

public:
  IloAllocationStack();
  ~IloAllocationStack();
  IloInt getSize() const { return _size; }
  IloInt getIndex() const {return _topIndex;}
  IloInt getFirstIndex() const {
    return _firstPart->_firstIndex;
  }
  void check(IloUInt size);
  IloAny checkPop();
  IloAny pop(){ 
    _topIndex --;
    return *(--_top);   
  }
  IloAny* pop(IloUInt);
  void checkPush(IloAny value);
  void push(IloAny value){
    _topIndex++;
    *(_top++) = value;
  }
  void push(IloUInt value){
    _topIndex++;
    *(_top++) = (IloAny)value;
  }
  IloAny* push(IloUInt size, IloAny* value); 
  void setIndex (IloInt); 
  IloAny allocate(IloUInt size);  // size is the size in Bytes 
  void free(IloAny p, const IloUInt s);
  void deallocate (Part*);
  void reset();
public:
  class NullIndex : public IloException {
  public:
    NullIndex();
    void print(ILOSTD(ostream)& out) const;
  };
  class StackingFull : public IloException {
  public:
    StackingFull();
    void print(ILOSTD(ostream)& out) const;
  };
  class UnstackingEmpty : public IloException {
  public:
    UnstackingEmpty();
    void print(ILOSTD(ostream)& out) const;
  };
};

void* operator new (size_t s, IloAllocationStack* heap);

#if defined (ILONEWOPERATOR)
inline void* operator new[] (size_t s, IloAllocationStack* heap){
  return operator new (s, heap);
}
#endif

#if defined(ILODELETEOPERATOR)
inline void operator delete(void*, size_t, IloAllocationStack*){}
inline void operator delete(void*, IloAllocationStack*){}
# if defined(ILONEWOPERATOR)
inline void operator delete [] (void*, IloAllocationStack*){}
# endif
#endif


#endif
